@@ -1,14 +1,11 @@
 # -*- coding: utf-8 -*-
-"""
-    celery.apps.worker
-    ~~~~~~~~~~~~~~~~~~
+"""Worker command-line program.
 
-    This module is the 'program-version' of :mod:`celery.worker`.
-
-    It does everything necessary to run that module
-    as an actual application, like installing signal handlers,
-    platform tweaks, and so on.
+This module is the 'program-version' of :mod:`celery.worker`.
 
+It does everything necessary to run that module
+as an actual application, like installing signal handlers,
+platform tweaks, and so on.
 """
 from __future__ import absolute_import, print_function, unicode_literals
 
@@ -16,22 +13,24 @@ import logging
 import os
 import platform as _platform
 import sys
-import warnings
 
+from datetime import datetime
 from functools import partial
 
-from billiard import current_process
+from billiard.process import current_process
 from kombu.utils.encoding import safe_str
 
-from celery import VERSION_BANNER, platforms, signals
+from celery import VERSION_BANNER
+from celery import platforms
+from celery import signals
 from celery.app import trace
-from celery.exceptions import (
-    CDeprecationWarning, WorkerShutdown, WorkerTerminate,
-)
+from celery.exceptions import WorkerShutdown, WorkerTerminate
 from celery.five import string, string_t
 from celery.loaders.app import AppLoader
-from celery.platforms import check_privileges
-from celery.utils import cry, isatty
+from celery.platforms import EX_FAILURE, EX_OK, check_privileges, isatty
+from celery.utils import static
+from celery.utils import term
+from celery.utils.debug import cry
 from celery.utils.imports import qualname
 from celery.utils.log import get_logger, in_sighandler, set_in_sighandler
 from celery.utils.text import pluralize
@@ -43,25 +42,6 @@ logger = get_logger(__name__)
 is_jython = sys.platform.startswith('java')
 is_pypy = hasattr(sys, 'pypy_version_info')
 
-W_PICKLE_DEPRECATED = """
-Starting from version 3.2 Celery will refuse to accept pickle by default.
-
-The pickle serializer is a security concern as it may give attackers
-the ability to execute any command.  It's important to secure
-your broker from unauthorized access when using pickle, so we think
-that enabling pickle should require a deliberate action and not be
-the default choice.
-
-If you depend on pickle then you should set a setting to disable this
-warning and to be sure that everything will continue working
-when you upgrade to Celery 3.2::
-
-    CELERY_ACCEPT_CONTENT = ['pickle', 'json', 'msgpack', 'yaml']
-
-You must only enable the serializers that you will actually use.
-
-"""
-
 
 def active_thread_count():
     from threading import enumerate
@@ -90,13 +70,14 @@ ARTLINES = [
 BANNER = """\
 {hostname} v{version}
 
-{platform}
+{platform} {timestamp}
 
 [config]
 .> app:         {app}
 .> transport:   {conninfo}
 .> results:     {results}
 .> concurrency: {concurrency}
+.> task events: {events}
 
 [queues]
 {queues}
@@ -109,9 +90,11 @@ EXTRA_INFO_FMT = """
 
 
 class Worker(WorkController):
+    """Worker as a program."""
 
-    def on_before_init(self, **kwargs):
-        trace.setup_worker_optimizations(self.app)
+    def on_before_init(self, quiet=False, **kwargs):
+        self.quiet = quiet
+        trace.setup_worker_optimizations(self.app, self.hostname)
 
         # this signal can be used to set up configuration for
         # workers by name.
@@ -119,17 +102,15 @@ class Worker(WorkController):
             sender=self.hostname, instance=self,
             conf=self.app.conf, options=kwargs,
         )
-        check_privileges(self.app.conf.CELERY_ACCEPT_CONTENT)
+        check_privileges(self.app.conf.accept_content)
 
     def on_after_init(self, purge=False, no_color=None,
                       redirect_stdouts=None, redirect_stdouts_level=None,
                       **kwargs):
-        self.redirect_stdouts = self._getopt(
-            'redirect_stdouts', redirect_stdouts,
-        )
-        self.redirect_stdouts_level = self._getopt(
-            'redirect_stdouts_level', redirect_stdouts_level,
-        )
+        self.redirect_stdouts = self.app.either(
+            'worker_redirect_stdouts', redirect_stdouts)
+        self.redirect_stdouts_level = self.app.either(
+            'worker_redirect_stdouts_level', redirect_stdouts_level)
         super(Worker, self).setup_defaults(**kwargs)
         self.purge = purge
         self.no_color = no_color
@@ -143,38 +124,44 @@ class Worker(WorkController):
         self._custom_logging = self.setup_logging()
         # apply task execution optimizations
         # -- This will finalize the app!
-        trace.setup_worker_optimizations(self.app)
+        trace.setup_worker_optimizations(self.app, self.hostname)
 
     def on_start(self):
-        if not self._custom_logging and self.redirect_stdouts:
-            self.app.log.redirect_stdouts(self.redirect_stdouts_level)
-
+        app = self.app
         WorkController.on_start(self)
 
-        # this signal can be used to e.g. change queues after
+        # this signal can be used to, for example, change queues after
         # the -Q option has been applied.
         signals.celeryd_after_setup.send(
-            sender=self.hostname, instance=self, conf=self.app.conf,
+            sender=self.hostname, instance=self, conf=app.conf,
         )
 
-        if not self.app.conf.value_set_for('CELERY_ACCEPT_CONTENT'):
-            warnings.warn(CDeprecationWarning(W_PICKLE_DEPRECATED))
-
         if self.purge:
             self.purge_messages()
 
-        # Dump configuration to screen so we have some basic information
-        # for when users sends bug reports.
-        print(safe_str(''.join([
-            string(self.colored.cyan(' \n', self.startup_info())),
-            string(self.colored.reset(self.extra_info() or '')),
-        ])), file=sys.__stdout__)
+        if not self.quiet:
+            self.emit_banner()
+
         self.set_process_status('-active-')
         self.install_platform_tweaks(self)
+        if not self._custom_logging and self.redirect_stdouts:
+            app.log.redirect_stdouts(self.redirect_stdouts_level)
+
+    def emit_banner(self):
+        # Dump configuration to screen so we have some basic information
+        # for when users sends bug reports.
+        use_image = term.supports_images()
+        if use_image:
+            print(term.imgcat(static.logo()))
+        print(safe_str(''.join([
+            string(self.colored.cyan(
+                ' \n', self.startup_info(artlines=not use_image))),
+            string(self.colored.reset(self.extra_info() or '')),
+        ])), file=sys.__stdout__)
 
     def on_consumer_ready(self, consumer):
         signals.worker_ready.send(sender=consumer)
-        print('{0} ready.'.format(safe_str(self.hostname), ))
+        logger.info('%s ready.', safe_str(self.hostname))
 
     def setup_logging(self, colorize=None):
         if colorize is None and self.no_color is not None:
@@ -185,10 +172,11 @@ class Worker(WorkController):
         )
 
     def purge_messages(self):
-        count = self.app.control.purge()
-        if count:
-            print('purge: Erased {0} {1} from the queue.\n'.format(
-                count, pluralize(count, 'message')))
+        with self.app.connection_for_write() as connection:
+            count = self.app.control.purge(connection=connection)
+            if count:  # pragma: no cover
+                print('purge: Erased {0} {1} from the queue.\n'.format(
+                    count, pluralize(count, 'message')))
 
     def tasklist(self, include_builtins=True, sep='\n', int_='celery.'):
         return sep.join(
@@ -202,13 +190,13 @@ class Worker(WorkController):
             tasklist = self.tasklist(include_builtins=include_builtins)
             return EXTRA_INFO_FMT.format(tasks=tasklist)
 
-    def startup_info(self):
+    def startup_info(self, artlines=True):
         app = self.app
         concurrency = string(self.concurrency)
-        appr = '{0}:0x{1:x}'.format(app.main or '__main__', id(app))
+        appr = '{0}:{1:#x}'.format(app.main or '__main__', id(app))
         if not isinstance(app.loader, AppLoader):
             loader = qualname(app.loader)
-            if loader.startswith('celery.loaders'):
+            if loader.startswith('celery.loaders'):  # pragma: no cover
                 loader = loader[14:]
             appr += ' ({0})'.format(loader)
         if self.autoscale:
@@ -219,15 +207,16 @@ class Worker(WorkController):
             pool = pool.__module__
         concurrency += ' ({0})'.format(pool.split('.')[-1])
         events = 'ON'
-        if not self.send_events:
-            events = 'OFF (enable -E to monitor this worker)'
+        if not self.task_events:
+            events = 'OFF (enable -E to monitor tasks in this worker)'
 
         banner = BANNER.format(
             app=appr,
             hostname=safe_str(self.hostname),
+            timestamp=datetime.now().replace(microsecond=0),
             version=VERSION_BANNER,
             conninfo=self.app.connection().as_uri(),
-            results=self.app.conf.CELERY_RESULT_BACKEND or 'disabled',
+            results=self.app.backend.as_uri(),
             concurrency=concurrency,
             platform=safe_str(_platform.platform()),
             events=events,
@@ -235,26 +224,27 @@ class Worker(WorkController):
         ).splitlines()
 
         # integrate the ASCII art.
-        for i, x in enumerate(banner):
-            try:
-                banner[i] = ' '.join([ARTLINES[i], banner[i]])
-            except IndexError:
-                banner[i] = ' ' * 16 + banner[i]
+        if artlines:
+            for i, _ in enumerate(banner):
+                try:
+                    banner[i] = ' '.join([ARTLINES[i], banner[i]])
+                except IndexError:
+                    banner[i] = ' ' * 16 + banner[i]
         return '\n'.join(banner) + '\n'
 
     def install_platform_tweaks(self, worker):
         """Install platform specific tweaks and workarounds."""
-        if self.app.IS_OSX:
-            self.osx_proxy_detection_workaround()
+        if self.app.IS_macOS:
+            self.macOS_proxy_detection_workaround()
 
         # Install signal handler so SIGHUP restarts the worker.
         if not self._isatty:
             # only install HUP handler if detached from terminal,
             # so closing the terminal window doesn't restart the worker
             # into the background.
-            if self.app.IS_OSX:
-                # OS X can't exec from a process using threads.
-                # See http://github.com/celery/celery/issues#issue/152
+            if self.app.IS_macOS:
+                # macOS can't exec from a process using threads.
+                # See https://github.com/celery/celery/issues#issue/152
                 install_HUP_not_supported_handler(worker)
             else:
                 install_worker_restart_handler(worker)
@@ -264,8 +254,8 @@ class Worker(WorkController):
         install_cry_handler()
         install_rdb_handler()
 
-    def osx_proxy_detection_workaround(self):
-        """See http://github.com/celery/celery/issues#issue/161"""
+    def macOS_proxy_detection_workaround(self):
+        """See https://github.com/celery/celery/issues#issue/161."""
         os.environ.setdefault('celery_dummy_proxy', 'set_by_celeryd')
 
     def set_process_status(self, info):
@@ -277,8 +267,7 @@ class Worker(WorkController):
 
 
 def _shutdown_handler(worker, sig='TERM', how='Warm',
-                      exc=WorkerShutdown, callback=None):
-
+                      exc=WorkerShutdown, callback=None, exitcode=EX_OK):
     def _handle_request(*args):
         with in_sighandler():
             from celery.worker import state
@@ -288,9 +277,9 @@ def _shutdown_handler(worker, sig='TERM', how='Warm',
                 safe_say('worker: {0} shutdown (MainProcess)'.format(how))
             if active_thread_count() > 1:
                 setattr(state, {'Warm': 'should_stop',
-                                'Cold': 'should_terminate'}[how], True)
+                                'Cold': 'should_terminate'}[how], exitcode)
             else:
-                raise exc()
+                raise exc(exitcode)
     _handle_request.__name__ = str('worker_{0}'.format(how))
     platforms.signals[sig] = _handle_request
 install_worker_term_handler = partial(
@@ -299,6 +288,7 @@ install_worker_term_handler = partial(
 if not is_jython:  # pragma: no cover
     install_worker_term_hard_handler = partial(
         _shutdown_handler, sig='SIGQUIT', how='Cold', exc=WorkerTerminate,
+        exitcode=EX_FAILURE,
     )
 else:  # pragma: no cover
     install_worker_term_handler = \
@@ -310,10 +300,11 @@ def on_SIGINT(worker):
     install_worker_term_hard_handler(worker, sig='SIGINT')
 if not is_jython:  # pragma: no cover
     install_worker_int_handler = partial(
-        _shutdown_handler, sig='SIGINT', callback=on_SIGINT
+        _shutdown_handler, sig='SIGINT', callback=on_SIGINT,
+        exitcode=EX_FAILURE,
     )
 else:  # pragma: no cover
-    def install_worker_int_handler(*a, **kw):
+    def install_worker_int_handler(*args, **kwargs):
         pass
 
 
@@ -333,7 +324,7 @@ def install_worker_restart_handler(worker, sig='SIGHUP'):
         import atexit
         atexit.register(_reload_current_worker)
         from celery.worker import state
-        state.should_stop = True
+        state.should_stop = EX_OK
     platforms.signals[sig] = restart_worker_sig_handler
 
 
@@ -343,7 +334,7 @@ def install_cry_handler(sig='SIGUSR1'):
         return
 
     def cry_handler(*args):
-        """Signal handler logging the stacktrace of all active threads."""
+        """Signal handler logging the stack-trace of all active threads."""
         with in_sighandler():
             safe_say(cry())
     platforms.signals[sig] = cry_handler
