@@ -1,27 +1,25 @@
-from __future__ import absolute_import
+"""Django-specific customization."""
+from __future__ import absolute_import, unicode_literals
 
 import os
 import sys
 import warnings
 
-from kombu.utils import cached_property, symbol_by_name
+from kombu.utils.imports import symbol_by_name
+from kombu.utils.objects import cached_property
 
 from datetime import datetime
 from importlib import import_module
 
+from celery import _state
 from celery import signals
-from celery.exceptions import FixupWarning
-
-if sys.version_info[0] < 3 and not hasattr(sys, 'pypy_version_info'):
-    from StringIO import StringIO
-else:
-    from io import StringIO
+from celery.exceptions import FixupWarning, ImproperlyConfigured
 
 __all__ = ['DjangoFixup', 'fixup']
 
 ERR_NOT_INSTALLED = """\
 Environment variable DJANGO_SETTINGS_MODULE is defined
-but Django is not installed.  Will not apply Django fixups!
+but Django isn't installed.  Won't apply Django fix-ups!
 """
 
 
@@ -33,7 +31,13 @@ def _maybe_close_fd(fh):
         pass
 
 
+def _verify_django_version(django):
+    if django.VERSION < (1, 8):
+        raise ImproperlyConfigured('Celery 4.x requires Django 1.8 or later.')
+
+
 def fixup(app, env='DJANGO_SETTINGS_MODULE'):
+    """Install Django fixup if settings module environment is set."""
     SETTINGS_MODULE = os.environ.get(env)
     if SETTINGS_MODULE and 'django' not in app.loader_cls.lower():
         try:
@@ -41,33 +45,40 @@ def fixup(app, env='DJANGO_SETTINGS_MODULE'):
         except ImportError:
             warnings.warn(FixupWarning(ERR_NOT_INSTALLED))
         else:
+            _verify_django_version(django)
             return DjangoFixup(app).install()
 
 
 class DjangoFixup(object):
+    """Fixup installed when using Django."""
 
     def __init__(self, app):
         self.app = app
-        self.app.set_default()
+        if _state.default_app is None:
+            self.app.set_default()
         self._worker_fixup = None
 
     def install(self):
         # Need to add project directory to path
         sys.path.append(os.getcwd())
 
+        self._settings = symbol_by_name('django.conf:settings')
         self.app.loader.now = self.now
-        self.app.loader.mail_admins = self.mail_admins
 
         signals.import_modules.connect(self.on_import_modules)
         signals.worker_init.connect(self.on_worker_init)
         return self
 
-    @cached_property
+    @property
     def worker_fixup(self):
         if self._worker_fixup is None:
             self._worker_fixup = DjangoWorkerFixup(self.app)
         return self._worker_fixup
 
+    @worker_fixup.setter
+    def worker_fixup(self, value):
+        self._worker_fixup = value
+
     def on_import_modules(self, **kwargs):
         # call django.setup() before task modules are imported
         self.worker_fixup.validate_models()
@@ -78,19 +89,13 @@ class DjangoFixup(object):
     def now(self, utc=False):
         return datetime.utcnow() if utc else self._now()
 
-    def mail_admins(self, subject, body, fail_silently=False, **kwargs):
-        return self._mail_admins(subject, body, fail_silently=fail_silently)
-
-    @cached_property
-    def _mail_admins(self):
-        return symbol_by_name('django.core.mail:mail_admins')
+    def autodiscover_tasks(self):
+        from django.apps import apps
+        return [config.name for config in apps.get_app_configs()]
 
     @cached_property
     def _now(self):
-        try:
-            return symbol_by_name('django.utils.timezone:now')
-        except (AttributeError, ImportError):  # pre django-1.4
-            return datetime.now
+        return symbol_by_name('django.utils.timezone:now')
 
 
 class DjangoWorkerFixup(object):
@@ -103,80 +108,19 @@ class DjangoWorkerFixup(object):
         self._cache = import_module('django.core.cache')
         self._settings = symbol_by_name('django.conf:settings')
 
-        # Database-related exceptions.
-        DatabaseError = symbol_by_name('django.db:DatabaseError')
-        try:
-            import MySQLdb as mysql
-            _my_database_errors = (mysql.DatabaseError,
-                                   mysql.InterfaceError,
-                                   mysql.OperationalError)
-        except ImportError:
-            _my_database_errors = ()      # noqa
-        try:
-            import psycopg2 as pg
-            _pg_database_errors = (pg.DatabaseError,
-                                   pg.InterfaceError,
-                                   pg.OperationalError)
-        except ImportError:
-            _pg_database_errors = ()      # noqa
-        try:
-            import sqlite3
-            _lite_database_errors = (sqlite3.DatabaseError,
-                                     sqlite3.InterfaceError,
-                                     sqlite3.OperationalError)
-        except ImportError:
-            _lite_database_errors = ()    # noqa
-        try:
-            import cx_Oracle as oracle
-            _oracle_database_errors = (oracle.DatabaseError,
-                                       oracle.InterfaceError,
-                                       oracle.OperationalError)
-        except ImportError:
-            _oracle_database_errors = ()  # noqa
-
-        try:
-            self._close_old_connections = symbol_by_name(
-                'django.db:close_old_connections',
-            )
-        except (ImportError, AttributeError):
-            self._close_old_connections = None
-        self.database_errors = (
-            (DatabaseError, ) +
-            _my_database_errors +
-            _pg_database_errors +
-            _lite_database_errors +
-            _oracle_database_errors
+        self.interface_errors = (
+            symbol_by_name('django.db.utils.InterfaceError'),
         )
+        self.DatabaseError = symbol_by_name('django.db:DatabaseError')
+
+    def django_setup(self):
+        import django
+        django.setup()
 
     def validate_models(self):
-        import django
-        try:
-            django_setup = django.setup
-        except AttributeError:
-            pass
-        else:
-            django_setup()
-        s = StringIO()
-        try:
-            from django.core.management.validation import get_validation_errors
-        except ImportError:
-            from django.core.management.base import BaseCommand
-            cmd = BaseCommand()
-            try:
-                # since django 1.5
-                from django.core.management.base import OutputWrapper
-                cmd.stdout = OutputWrapper(sys.stdout)
-                cmd.stderr = OutputWrapper(sys.stderr)
-            except ImportError:
-                cmd.stdout, cmd.stderr = sys.stdout, sys.stderr
-
-            cmd.check()
-        else:
-            num_errors = get_validation_errors(s, None)
-            if num_errors:
-                raise RuntimeError(
-                    'One or more Django models did not validate:\n{0}'.format(
-                        s.getvalue()))
+        from django.core.checks import run_checks
+        self.django_setup()
+        run_checks()
 
     def install(self):
         signals.beat_embedded_init.connect(self.close_database)
@@ -202,18 +146,20 @@ class DjangoWorkerFixup(object):
         # the inherited DB conn to also get broken in the parent
         # process so we need to remove it without triggering any
         # network IO that close() might cause.
-        try:
-            for c in self._db.connections.all():
-                if c and c.connection:
-                    _maybe_close_fd(c.connection)
-        except AttributeError:
-            if self._db.connection and self._db.connection.connection:
-                _maybe_close_fd(self._db.connection.connection)
+        for c in self._db.connections.all():
+            if c and c.connection:
+                self._maybe_close_db_fd(c.connection)
 
         # use the _ version to avoid DB_REUSE preventing the conn.close() call
         self._close_database()
         self.close_cache()
 
+    def _maybe_close_db_fd(self, fd):
+        try:
+            _maybe_close_fd(fd)
+        except self.interface_errors:
+            pass
+
     def on_task_prerun(self, sender, **kwargs):
         """Called before every task."""
         if not getattr(sender.request, 'is_eager', False):
@@ -227,8 +173,6 @@ class DjangoWorkerFixup(object):
             self.close_cache()
 
     def close_database(self, **kwargs):
-        if self._close_old_connections:
-            return self._close_old_connections()  # Django 1.6
         if not self.db_reuse_max:
             return self._close_database()
         if self._db_recycles >= self.db_reuse_max * 2:
@@ -237,19 +181,12 @@ class DjangoWorkerFixup(object):
         self._db_recycles += 1
 
     def _close_database(self):
-        try:
-            funs = [conn.close for conn in self._db.connections.all()]
-        except AttributeError:
-            if hasattr(self._db, 'close_old_connections'):  # django 1.6
-                funs = [self._db.close_old_connections]
-            else:
-                # pre multidb, pending deprication in django 1.6
-                funs = [self._db.close_connection]
-
-        for close in funs:
+        for conn in self._db.connections.all():
             try:
-                close()
-            except self.database_errors as exc:
+                conn.close()
+            except self.interface_errors:
+                pass
+            except self.DatabaseError as exc:
                 str_exc = str(exc)
                 if 'closed' not in str_exc and 'not connected' not in str_exc:
                     raise
