@@ -1,14 +1,7 @@
 # -*- coding: utf-8 -*-
-"""
-    celery.loaders.base
-    ~~~~~~~~~~~~~~~~~~~
+"""Loader base class."""
+from __future__ import absolute_import, unicode_literals
 
-    Loader base class.
-
-"""
-from __future__ import absolute_import
-
-import anyjson
 import imp as _imp
 import importlib
 import os
@@ -17,12 +10,11 @@ import sys
 
 from datetime import datetime
 
-from kombu.utils import cached_property
-from kombu.utils.encoding import safe_str
+from kombu.utils import json
 
 from celery import signals
-from celery.datastructures import DictAttribute, force_mapping
 from celery.five import reraise, string_t
+from celery.utils.collections import DictAttribute, force_mapping
 from celery.utils.functional import maybe_list
 from celery.utils.imports import (
     import_from_cwd, symbol_by_name, NotAPackage, find_module,
@@ -31,6 +23,7 @@ from celery.utils.imports import (
 __all__ = ['BaseLoader']
 
 _RACE_PROTECTION = False
+
 CONFIG_INVALID_NAME = """\
 Error: Module '{module}' doesn't exist, or it's not a valid \
 Python module name.
@@ -40,9 +33,11 @@ CONFIG_WITH_SUFFIX = CONFIG_INVALID_NAME + """\
 Did you mean '{suggest}'?
 """
 
+unconfigured = object()
+
 
 class BaseLoader(object):
-    """The base class for loaders.
+    """Base class for loaders.
 
     Loaders handles,
 
@@ -58,14 +53,14 @@ class BaseLoader(object):
             See :meth:`on_worker_shutdown`.
 
         * What modules are imported to find tasks?
-
     """
+
     builtin_modules = frozenset()
     configured = False
     override_backends = {}
     worker_initialized = False
 
-    _conf = None
+    _conf = unconfigured
 
     def __init__(self, app, **kwargs):
         self.app = app
@@ -77,25 +72,23 @@ class BaseLoader(object):
         return datetime.now()
 
     def on_task_init(self, task_id, task):
-        """This method is called before a task is executed."""
+        """Called before a task is executed."""
         pass
 
     def on_process_cleanup(self):
-        """This method is called after a task is executed."""
+        """Called after a task is executed."""
         pass
 
     def on_worker_init(self):
-        """This method is called when the worker (:program:`celery worker`)
-        starts."""
+        """Called when the worker (:program:`celery worker`) starts."""
         pass
 
     def on_worker_shutdown(self):
-        """This method is called when the worker (:program:`celery worker`)
-        shuts down."""
+        """Called when the worker (:program:`celery worker`) shuts down."""
         pass
 
     def on_worker_process_init(self):
-        """This method is called when a child process starts."""
+        """Called when a child process starts."""
         pass
 
     def import_task_module(self, module):
@@ -117,8 +110,8 @@ class BaseLoader(object):
         return [
             self.import_task_module(m) for m in (
                 tuple(self.builtin_modules) +
-                tuple(maybe_list(self.app.conf.CELERY_IMPORTS)) +
-                tuple(maybe_list(self.app.conf.CELERY_INCLUDE))
+                tuple(maybe_list(self.app.conf.imports)) +
+                tuple(maybe_list(self.app.conf.include))
             )
         ]
 
@@ -148,12 +141,12 @@ class BaseLoader(object):
     def _smart_import(self, path, imp=None):
         imp = self.import_module if imp is None else imp
         if ':' in path:
-            # Path includes attribute so can just jump here.
-            # e.g. ``os.path:abspath``.
+            # Path includes attribute so can just jump
+            # here (e.g., ``os.path:abspath``).
             return symbol_by_name(path, imp=imp)
 
         # Not sure if path is just a module name or if it includes an
-        # attribute name (e.g. ``os.path``, vs, ``os.path.abspath``).
+        # attribute name (e.g., ``os.path``, vs, ``os.path.abspath``).
         try:
             return imp(path)
         except ImportError:
@@ -178,29 +171,27 @@ class BaseLoader(object):
     def cmdline_config_parser(
             self, args, namespace='celery',
             re_type=re.compile(r'\((\w+)\)'),
-            extra_types={'json': anyjson.loads},
+            extra_types={'json': json.loads},
             override_types={'tuple': 'json',
                             'list': 'json',
                             'dict': 'json'}):
         from celery.app.defaults import Option, NAMESPACES
-        namespace = namespace.upper()
+        namespace = namespace and namespace.lower()
         typemap = dict(Option.typemap, **extra_types)
 
         def getarg(arg):
-            """Parse a single configuration definition from
-            the command-line."""
-
+            """Parse single configuration from command-line."""
             # ## find key/value
             # ns.key=value|ns_key=value (case insensitive)
             key, value = arg.split('=', 1)
-            key = key.upper().replace('.', '_')
+            key = key.lower().replace('.', '_')
 
-            # ## find namespace.
-            # .key=value|_key=value expands to default namespace.
+            # ## find name-space.
+            # .key=value|_key=value expands to default name-space.
             if key[0] == '_':
                 ns, key = namespace, key[1:]
             else:
-                # find namespace part of key
+                # find name-space part of key
                 ns, key = key.split('_', 1)
 
             ns_key = (ns and ns + '_' or '') + key
@@ -214,27 +205,13 @@ class BaseLoader(object):
                 value = typemap[type_](value)
             else:
                 try:
-                    value = NAMESPACES[ns][key].to_python(value)
+                    value = NAMESPACES[ns.lower()][key].to_python(value)
                 except ValueError as exc:
                     # display key name in error message.
                     raise ValueError('{0!r}: {1}'.format(ns_key, exc))
             return ns_key, value
         return dict(getarg(arg) for arg in args)
 
-    def mail_admins(self, subject, body, fail_silently=False,
-                    sender=None, to=None, host=None, port=None,
-                    user=None, password=None, timeout=None,
-                    use_ssl=False, use_tls=False, charset='utf-8'):
-        message = self.mail.Message(sender=sender, to=to,
-                                    subject=safe_str(subject),
-                                    body=safe_str(body),
-                                    charset=charset)
-        mailer = self.mail.Mailer(host=host, port=port,
-                                  user=user, password=password,
-                                  timeout=timeout, use_ssl=use_ssl,
-                                  use_tls=use_tls)
-        mailer.send(message, fail_silently=fail_silently)
-
     def read_configuration(self, env='CELERY_CONFIG_MODULE'):
         try:
             custom_config = os.environ[env]
@@ -244,7 +221,6 @@ class BaseLoader(object):
             if custom_config:
                 usercfg = self._import_config_module(custom_config)
                 return DictAttribute(usercfg)
-        return {}
 
     def autodiscover_tasks(self, packages, related_name='tasks'):
         self.task_modules.update(
@@ -254,14 +230,10 @@ class BaseLoader(object):
     @property
     def conf(self):
         """Loader configuration."""
-        if self._conf is None:
+        if self._conf is unconfigured:
             self._conf = self.read_configuration()
         return self._conf
 
-    @cached_property
-    def mail(self):
-        return self.import_module('celery.utils.mail')
-
 
 def autodiscover_tasks(packages, related_name='tasks'):
     global _RACE_PROTECTION
@@ -276,15 +248,15 @@ def autodiscover_tasks(packages, related_name='tasks'):
 
 
 def find_related_module(package, related_name):
-    """Given a package name and a module name, tries to find that
-    module."""
-
+    """Find module in package."""
     # Django 1.7 allows for speciying a class name in INSTALLED_APPS.
     # (Issue #2248).
     try:
         importlib.import_module(package)
     except ImportError:
         package, _, _ = package.rpartition('.')
+        if not package:
+            raise
 
     try:
         pkg_path = importlib.import_module(package).__path__
